<template>
</template>
<style id="domino-css">
  .domino-plotly-auto-height .js-plotly-plot {
    height: 100%;
  }

  .domino-card-grid .modebar-container {
    display: none;
  }
  .dca-transform-header {
    background-color: var(--jp-border-color3);
  }
  .milkdown-menu .button.active {
    background-color: var(--jp-border-color2);
  }

  .domino-thumbnail .js-plotly-plot {
    height: 100%;
  }
  .domino-thumbnail .js-plotly-plot .plot-container.plotly {
    height: 100%;
  }
  .domino-ml-model-type-button {
    cursor: pointer;
  }
  .domino-ml-model-type-button:hover {
    background-color: var(--jp-border-color2);
  }
  .dca-ml-task-selection .v-stepper__content,
  .vuetify-styles .v-dialog>.dca-ml-task-selection.v-card .v-card__text {
    padding: 0
  }
  .dca-automl-config .v-expansion-panel-header .v-chip {
    overflow: unset;
  }
  .vuetify-styles .dca-hyperparameters-checkbox.v-input--selection-controls {
    margin: 0;
  }
  .dca-hyperparameters-list {
    transition: opacity 0.5s;
  }
  .dca-hyperparameters-list-off {
    opacity: 0.5;
    pointer-events:none;
  }
</style>
